#include "color.h"
/*!
\addtogroup color_functions Color Functions
*/
/*+----------------------------------------------------------------------------------------------+*/
/*!
\ingroup color_functions
\brief Convert RGB to YCbYCr format

\param R
   Red amount (0..255)
\param G
   Green amount (0..255)
\param B
   Blue amount (0..255)

\return
   The specified RGB color packed as <b>YCbYCr</b> format.


Use this function to convert RGB values into <b>YCbYCr</b> format. (YCbYCr is also known as UYVY) \n

\bug This doesnt work on hardware (yet) returns always the same value (black).
*/
/*
   Author: Peter (www.console-dev.de)
   Date..: 21th June 2003
*/
u32 COLOR_ConvertRgb2YCbYCr(u8 R, u8 G, u8 B)
{
#ifndef DOXYGEN_SHOULD_SKIP_THIS
   #ifndef ABS
      #define ABS(n)    ((((n)<0)?(n)*-1:(n)))
   #endif /* ABS */
#endif /* DOXYGEN_SHOULD_SKIP_THIS */

   u8 Y, Cb, Cr;

   Y  = ABS((77/256)*R +  (150/256)*G +  (29/256)*B);
   Cb = ABS(-(44/256)*R -  (87/256)*G + (131/256)*B + 128);
   Cr = ABS((131/256)*R - (110/256)*G -  (21/256)*B + 128);

   return (Y << 24) | (Cb << 16) | (Y << 8) | Cr;
}
/*+----------------------------------------------------------------------------------------------+*/
/*!
\ingroup color_functions
\brief Get red amount from a YCbYCr color

\param YCbYCr
   The color to get the red value from in <b>YCbYCr</b> format

\return
   The <b>red</b> amount. It's in range between 0..252

Use this function to get the red amount from a color specified in YCbYCr format

\todo Not tested yet

\sa
   COLOR_ConvertRgb2YCbYCr() \n
   COLOR_GetGValue() \n
   COLOR_GetBValue()
*/
/*
   Author: Peter (www.console-dev.de)
   Date..: 21th June 2003
*/
u8 COLOR_GetRValue(u32 YCbYCr)
{
   u8 Y, Cr;

   Y  = (YCbYCr & 0x0000ffff) >> 8;
   Cr = (YCbYCr & 0x000000ff);

   // R = Y + 1.371(Cr - 128)
   return (u8)(Y + (1.371f * (Cr - 128)));
}
/*+----------------------------------------------------------------------------------------------+*/
/*!
\ingroup color_functions
\brief Get green amount from a YCbYCr color

\param YCbYCr
   The color to get the green value from in <b>YCbYCr</b> format

\return
   The <b>green</b> amount. It's in range between 0..252

Use this function to get the red amount from a color specified in YCbYCr format

\todo Not tested yet

\sa
   COLOR_ConvertRgb2YCbYCr() \n
   COLOR_GetRValue() \n
   COLOR_GetBValue()
*/
/*
   Author: Peter (www.console-dev.de)
   Date..: 21th June 2003
*/
u8 COLOR_GetGValue(u32 YCbYCr)
{
   u8 Y, Cb, Cr;

   Cb = (YCbYCr & 0x00ffffff) >>16;
   Y  = (YCbYCr & 0x0000ffff) >> 8;
   Cr = (YCbYCr & 0x000000ff);

   // G = Y - 0.698(Cr - 128) - 0.336(Cb - 128)
   return (u8)(Y - 0.698f*(Cr - 128) - 0.336f*(Cb - 128));
}
/*+----------------------------------------------------------------------------------------------+*/
/*!
\ingroup color_functions
\brief Get blue amount from a YCbYCr color

\param YCbYCr
   The color to get the blue value from in <b>YCbYCr</b> format

\return
   The <b>blue</b> amount. It's in range between 0..248

Use this function to get the blue amount from a color specified in YCbYCr format

\todo Not tested yet

\sa
   COLOR_ConvertRgb2YCbYCr() \n
   COLOR_GetRValue() \n
   COLOR_GetGValue()
*/
/*
   Author: Peter (www.console-dev.de)
   Date..: 21th June 2003
*/
u8 COLOR_GetBValue(u32 YCbYCr)
{
   u8 Y, Cb;

   Y  = (YCbYCr) >> 24;
   Cb = (YCbYCr & 0x00ffffff) >>16;

   // B = Y + 1.732(Cb - 128)
   return (u8)(Y + 1.732f*(Cb - 128));
}
/*+----------------------------------------------------------------------------------------------+*/







/* END OF FILE */
