#include "video.h"
/*!
\addtogroup video_functions Video Functions
*/
#ifndef DOXYGEN_SHOULD_SKIP_THIS
/*+----------------------------------------------------------------------------------------------+*/
// VIDEO_640X480_NTSC_YUV16
static const u32 VIDEO_Mode640X480NtscYUV16[32] = {
0x0F060001, 0x476901AD, 0x02EA5140, 0x00030018,
0x00020019, 0x410C410C, 0x40ED40ED, 0x00435A4E,
0x00000000, 0x00435A4E, 0x00000000, 0x00000000,
0x110701AE, 0x10010001, 0x00010001, 0x00010001,
0x00000000, 0x00000000, 0x28500100, 0x1AE771F0,
0x0DB4A574, 0x00C1188E, 0xC4C0CBE2, 0xFCECDECF,
0x13130F08, 0x00080C0F, 0x00FF0000, 0x00000000,
0x02800000, 0x000000FF, 0x00FF00FF, 0x00FF00FF};
/*+----------------------------------------------------------------------------------------------+*/
// VIDEO_640X480_PAL50_YUV16
static const u32 VIDEO_Mode640X480Pal50YUV16[32] = {
0x11F50101, 0x4B6A01B0, 0x02F85640, 0x00010023,
0x00000024, 0x4D2B4D6D, 0x4D8A4D4C, 0x00435A4E,
0x00000000, 0x00435A4E, 0x00000000, 0x013C0144,
0x113901B1, 0x10010001, 0x00010001, 0x00010001,
0x00000000, 0x00000000, 0x28500100, 0x1AE771F0,
0x0DB4A574, 0x00C1188E, 0xC4C0CBE2, 0xFCECDECF,
0x13130F08, 0x00080C0F, 0x00FF0000, 0x00000000,
0x02800000, 0x000000FF, 0x00FF00FF, 0x00FF00FF};
/*+----------------------------------------------------------------------------------------------+*/
// VIDEO_640X480_PAL60_YUV16
static const u32 VIDEO_Mode640X480Pal60YUV16[32] = {
0x0F060001, 0x476901AD, 0x02EA5140, 0x00030018,
0x00020019, 0x410C410C, 0x40ED40ED, 0x00435A4E,
0x00000000, 0x00435A4E, 0x00000000, 0x00050176,
0x110701AE, 0x10010001, 0x00010001, 0x00010001,
0x00000000, 0x00000000, 0x28500100, 0x1AE771F0,
0x0DB4A574, 0x00C1188E, 0xC4C0CBE2, 0xFCECDECF,
0x13130F08, 0x00080C0F, 0x00FF0000, 0x00000000,
0x02800000, 0x000000FF, 0x00FF00FF, 0x00FF00FF};
/*+----------------------------------------------------------------------------------------------+*/
#endif /* DOXYGEN_SHOULD_SKIP_THIS */
/*!
\ingroup video_functions

\brief Initialize the Video Interface

\param VideoMode
   The video mode you want to initialize. \n\n

   This can be one of these defines:
      - ::VIDEO_640X480_NTSC_YUV16
      - ::VIDEO_640X480_PAL50_YUV16
      - ::VIDEO_640X480_PAL60_YUV16
   \n\n

   Where it is in the following format: \n
      - VIDEO_[Width]X[Height]_[VideoStandard]_[ColorFormat] \n\n

   Explanation of different video standards:
      - PAL \n
           <b>P</b>hase <b>A</b>lternating <b>L</b>ine, is a video standard used in many countries around the world.
           Countries such as the United Kingdom, The Netherlands, Germany and more use PAL as their video standard.
           There are different types of PAL standards, one of which uses a vertical frequency of 60Hz and one of
           which uses a vertical frequency of 50Hz.

      - NTSC \n
           <b>N</b>ational <b>T</b>elevision <b>S</b>ystem <b>C</b>ommitee, is another video standard used
           in many countries around the world. While it's not technically superior, it's more widely used in countries
           such as the USA, Japan and many more. \n
           If a TV is PAL only, it will display NTSC pictures in black and white, due to the lack of colour signal.

   \n
   The fundamentals of video standards:

\code
   +----------------+------------+----------------+---------------------+
   | Video Standard | Frames/sec | Frequency (Hz) | Colour signal (MHz) |
   +----------------+------------+----------------+---------------------+
   |    NTSC        |    30      |   60           |    3.58             |
   |    PAL 50      |    25      |   50           |    4.43             |
   |    PAL 60      |    30      |   60           |    4.43             |
   +----------------+------------+----------------+---------------------+
\endcode
\n\n
   Video-standard informations found at: \n
      - http://www.horrordvds.com/extras/articles/palntsc
      - http://www.techtronics.com/uk/shop/87-00-video-standards-ntsc-pal.html
*/
/*
   Author: Titanik^CrazyNation

   Copied from his source and made it only a little bit more "accurate"
*/
void VIDEO_Init(u32 VideoMode)
{
   u32 Counter=0;
   u32 *pSrcAddr=NULL;
   u32 *pDstAddr=MEM_VIDEO_BASE_PTR;

   switch(VideoMode)
   {
      case VIDEO_640X480_NTSC_YUV16:
               pSrcAddr = (u32*)VIDEO_Mode640X480NtscYUV16;
               break;

      case VIDEO_640X480_PAL50_YUV16:
               pSrcAddr = (u32*)VIDEO_Mode640X480Pal50YUV16;
               break;

      case VIDEO_640X480_PAL60_YUV16:
               pSrcAddr = (u32*)VIDEO_Mode640X480Pal60YUV16;
               break;

      /* Use NTSC as default */
      default:
               pSrcAddr = (u32*)VIDEO_Mode640X480NtscYUV16;
               break;
    }

    for(Counter=0; Counter<32; Counter++)
    {
      pDstAddr[Counter] = pSrcAddr[Counter];
    }
}
/*+----------------------------------------------------------------------------------------------+*/
/*!
\ingroup video_functions

\brief Set framebuffer address

\param Which
   Which framebuffer you want to set the address for

   For the <b>Which</b> paramater, you can specify one of these defines: \n
      - ::VIDEO_FRAMEBUFFER_1
      - ::VIDEO_FRAMEBUFFER_2
      - ::VIDEO_FRAMEBUFFER_BOTH

\param FrameBufferAddr
   The memory address of the framebuffer

Any detailed information on this? \n
Email me if you know more! :)
*/
/*
   Author: Titanik^CrazyNation

   Copied from his source and made it only a little bit more "accurate"
*/
void VIDEO_SetFrameBuffer(u32 Which, u32 FrameBufferAddr)
{
   switch(Which)
   {
      /* Framebuffer 1 */
      case VIDEO_FRAMEBUFFER_1:
               R_VIDEO_FRAMEBUFFER_1 = FrameBufferAddr;
               break;

      /* Framebuffer 2 */
      case VIDEO_FRAMEBUFFER_2:
               R_VIDEO_FRAMEBUFFER_2 = FrameBufferAddr;
               break;

      /* Set both Framebuffers */
      case VIDEO_FRAMEBUFFER_BOTH:
               R_VIDEO_FRAMEBUFFER_1 = FrameBufferAddr;
               R_VIDEO_FRAMEBUFFER_2 = FrameBufferAddr;
               break;
   }
}
/*+----------------------------------------------------------------------------------------------+*/
/*!
\ingroup video_functions

\brief Wait for vertical retrace

   What is Vertical retrace (VSync)?
      - When the beam arrives the lower right corner, it goes to the upper left corner.
        This moment is called  vertical retrace.
*/
/*
   Author: Titanik^CrazyNation

   Copied from his source and made it only a little bit more "accurate".
*/
void VIDEO_WaitVSync(void)
{
   while(R_VIDEO_HALFLINE_1 >= 200) {}
   while(R_VIDEO_HALFLINE_1 <  200) {}
}
/* END OF FILE */
